# Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

# * The software is provided "as is" without any warranty, neither express
#   nor implied.
# * In no event will the author be held liable for any damage(s) arising
#   from the use of the software.
# * Redistribution of the software is allowed only in unmodified form.
# * Permission is granted to use the software solely for personal private
#   and non-commercial purposes.
# * An individuals use of the software in his or her capacity or function
#   as an agent, (independent) contractor, employee, member or officer of
#   a business, corporation or organization (commercial or non-commercial)
#   does not qualify as personal private and non-commercial purpose.
# * Without written approval from the author the software must not be used
#   for a business, for commercial, corporate, governmental, military or
#   organizational purposes of any kind, or in a commercial, corporate,
#   governmental, military or organizational environment of any kind.

_CL_=
CL=
LIBPATH=
LINK=
LOG_BUILD_COMMANDLINES=

.SUFFIXES:

all: myth0

clobber: clean
	ERASE MYTHS.DOS MYTH0.COM MYTH0.DLL MYTH0.EXE MYTH0.SCR

clean:
	ERASE MYTH0.EXP MYTH0.LIB MYTH0.OBJ MYTH0C.OBJ MYTH0D.OBJ MYTH0E.OBJ

MYTHS.OMF: MYTHS.MAK
	ML.EXE /Brepro /Fo$@ /nologo /omf /Ta<< /W3 /X
; Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

	.model	tiny

e_header struct				; DOS .EXE header
e_magic	word	5A4Dh			; magic number "MZ"
e_cblp	word	stop - header		; bytes on last page of file
e_cp	word	1			; pages in file
e_crlc	word	0			; number of relocation table entries
e_cparhdr word	sizeof header / 16	; size of header in paragraphs
e_minalloc word	0			; minimum extra paragraphs needed
e_maxalloc word	65535			; maximum extra paragraphs needed
e_ss	word	0			; initial (PSP relative) SS value
e_sp	word	stack - header		; initial (SS relative) SP value
e_csum	word	0			; checksum
e_ip	word	sizeof message		; initial (CS relative) IP value
e_cs	word	0			; initial (PSP relative) CS value
e_lfarlc word	sizeof header		; file address of relocation table
e_ovno	word	0			; overlay number
e_res	word	4 dup (0)		; reserved words
e_oemid	word	5719h			; OEM identifier
e_oeminfo word	2704h			; OEM information
e_res2	word	10 dup (0)		; reserved words
e_lfanew dword	0			; file address of new exe header
e_header ends

	.code

header	e_header <>

message	byte	"(C)opyright 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>", 0Dh, 0Ah, 07h, 24h

start:
	push	cs
	pop	ds
	xor	dx, dx
	mov	ah, 09h
	int	21h

	mov	ax, 4C01h
	int	21h
stop:
	dword	20 dup (?)
stack:
	end	start
<<NOKEEP

MYTHS.DOS: MYTHS.MAK
	CERTUTIL.EXE /DecodeHex /F /V << $@
4D 5A 90 00 01 00 00 00 04 00 00 00 FF FF 00 00   MZ..............
E0 00 00 00 43 00 00 00 40 00 00 00 00 00 00 00   ....C...@.......
00 00 00 00 19 57 04 27 00 00 00 00 00 00 00 00   .....W.'........
00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00   ................
28 43 29 6F 70 79 72 69 67 68 74 20 32 30 30 34   (C)opyright 2004
2D 32 30 32 34 2C 20 53 74 65 66 61 6E 20 4B 61   -2024, Stefan Ka
6E 74 68 61 6B 20 3C 73 74 65 66 61 6E 2E 6B 61   nthak <stefan.ka
6E 74 68 61 6B 40 6E 65 78 67 6F 2E 64 65 3E 0D   nthak@nexgo.de>.
0A 07 24 0E 1F 33 D2 B4 09 CD 21 B8 01 4C CD 21   .....3....!..L.!
<<NOKEEP

myth0: MYTHS.DOS MYTH0.DLL MYTH0.EXE MYTH0.COM MYTH0.SCR
	"$(MAKEDIR)\MYTH0.SCR"

MYTH0.SCR: MYTH0.OBJ # MYTHS.DOS
	LINK.EXE /LINK /BASE:65536 /BREPRO /DEFAULTLIB:KERNEL32.LIB /DEFAULTLIB:USER32.LIB /ENTRY:wmainCRTStartup /FIXED /LARGEADDRESSAWARE /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:6.0 /OUT:$@ /RELEASE /STUB:MYTHS.DOS /SUBSYSTEM:CONSOLE /SWAPRUN:CD,NET /VERSION:0.815 $**

MYTH0.OBJ: MYTHS.MAK
	CL.EXE /Brepro /c /Fo$@ /GAF /Gwy /nologo /O2isy /Tc<< /W4 /we4013 /Zl
// Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

// https://msdn.microsoft.com/en-us/library/aa383732.aspx

#define STRICT
#define UNICODE
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

// https://msdn.microsoft.com/en-us/library/aa364221.aspx

const	FILE_DISPOSITION_INFO	fdi = {TRUE};

// https://msdn.microsoft.com/en-us/library/aa364398.aspx

const	FILE_RENAME_INFO	fri = {TRUE, (HANDLE) NULL, sizeof(L''), L''};

// https://msdn.microsoft.com/en-us/library/hh916382.aspx

__declspec(safebuffers)
BOOL	CDECL	PrintConsole(HANDLE hConsole, [SA_FormatString(Style="printf")] LPCWSTR lpFormat, ...)
{
	WCHAR	szOutput[1024];
	DWORD	dwOutput;
	DWORD	dwConsole;

	// https://msdn.microsoft.com/en-us/library/kb57fad8.aspx

	va_list	vaInput;
	va_start(vaInput, lpFormat);

	// https://msdn.microsoft.com/en-us/library/ms647551.aspx

	dwOutput = wvsprintf(szOutput, lpFormat, vaInput);

	va_end(vaInput);

	if (dwOutput == 0UL)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/ms687401.aspx

	if (!WriteConsole(hConsole, szOutput, dwOutput, &dwConsole, NULL))
		return FALSE;

	return dwConsole == dwOutput;
}

// https://msdn.microsoft.com/en-us/library/f9t8842e.aspx

__declspec(noreturn)
VOID	CDECL	wmainCRTStartup(VOID)
{
	DWORD	dwError = ERROR_SUCCESS;
	DWORD	dwProcess;
	WCHAR	szProcess[MAX_PATH];
	HANDLE	hProcess;

	// https://msdn.microsoft.com/en-us/library/ms683231.aspx

	HANDLE	hConsole = GetStdHandle(STD_ERROR_HANDLE);

	if (hConsole == INVALID_HANDLE_VALUE)
		dwError = GetLastError();
	else
	{	// https://msdn.microsoft.com/en-us/library/ms683197.aspx

		dwProcess = GetModuleFileName((HMODULE) NULL,
		                              szProcess,
		                              sizeof(szProcess) / sizeof(*szProcess));
		if (dwProcess == 0UL)
			PrintConsole(hConsole,
			             L"GetModuleFileName() returned error %lu\n",
			             dwError = GetLastError());
		else
		{	// https://msdn.microsoft.com/en-us/library/aa363858.aspx

			hProcess = CreateFile(szProcess,
			                      DELETE,
			                      FILE_SHARE_DELETE,
			                      (LPSECURITY_ATTRIBUTES) NULL,
			                      OPEN_EXISTING,
			                      FILE_ATTRIBUTE_NORMAL,
			                      (HANDLE) NULL);

			if (hProcess == INVALID_HANDLE_VALUE)
				PrintConsole(hConsole,
				             L"CreateFile() returned error %lu\n",
				             dwError = GetLastError());
			else
			{	// https://msdn.microsoft.com/en-us/library/aa365539.aspx
				// https://msdn.microsoft.com/en-us/library/aa364228.aspx

				if (!SetFileInformationByHandle(hProcess,
#ifdef MYTHS
				                                FileDispositionInfo,
				                                &fdi,
				                                sizeof(fdi)))
#else
				                                FileRenameInfo,
				                                &fri,
				                                sizeof(fri)))
#endif
					PrintConsole(hConsole,
					             L"SetFileInformationByHandle() returned error %lu\n",
					             dwError = GetLastError());

				// https://msdn.microsoft.com/en-us/library/ms724211.aspx

				if (!CloseHandle(hProcess))
					PrintConsole(hConsole,
					             L"CloseHandle() returned error %lu\n",
					             GetLastError());
			}
		}

		// https://msdn.microsoft.com/en-us/library/ms724211.aspx

		if (!CloseHandle(hConsole))
			PrintConsole(hConsole,
			             L"CloseHandle() returned error %lu\n",
			             GetLastError());
	}

	// https://msdn.microsoft.com/en-us/library/ms682658.aspx

	ExitProcess(dwError);
}
<<NOKEEP

MYTH0.DLL: MYTH0D.OBJ # MYTHS.DOS
	LINK.EXE /LINK /BASE:65536 /BREPRO /DEFAULTLIB:KERNEL32.LIB /DLL /DYNAMICBASE /LARGEADDRESSAWARE /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:4.0 /OUT:$@ /RELEASE /STUB:MYTHS.DOS /SWAPRUN:CD,NET /VERSION:0.815 $**

MYTH0D.OBJ: MYTHS.MAK
	CL.EXE /Brepro /c /Fo$@ /GAF /Gwy /nologo /O2isy /Tc<< /W4 /we4013 /Zl
// Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

// https://msdn.microsoft.com/en-us/library/aa383732.aspx

#define STRICT
#define UNICODE
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

// https://msdn.microsoft.com/en-us/library/hh916382.aspx

__declspec(safebuffers)
BOOL	WINAPI	_DllMainCRTStartup(HMODULE hModule, DWORD dwReason, CONTEXT *lpContext)
{
	WCHAR	szBuffer[MAX_PATH];
	WCHAR	szModule[MAX_PATH];
	WCHAR	szProcess[MAX_PATH];

	if (dwReason != DLL_PROCESS_ATTACH)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/ms683197.aspx

	if (GetModuleFileName((HMODULE) NULL, szProcess, sizeof(szProcess) / sizeof(*szProcess)) == 0UL)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/ms683197.aspx

	if (GetModuleFileName(hModule, szModule, sizeof(szModule) / sizeof(*szModule)) == 0UL)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/aa364992.aspx

	if (GetTempPath(sizeof(szBuffer) / sizeof(*szBuffer), szBuffer) == 0U)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/aa364991.aspx

	if (GetTempFileName(szBuffer, L"dll", 0U, szBuffer) == 0U)
		return FALSE;
#ifdef MYTHS
	// https://msdn.microsoft.com/en-us/library/aa363915.aspx

	if (!DeleteFile(szBuffer))
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/aa365239.aspx

	if (!MoveFile(szModule, szBuffer))
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/aa363851.aspx

	if (!CopyFile(szProcess, szModule, FALSE))
		return FALSE;
#else
	// https://msdn.microsoft.com/en-us/library/aa365240.aspx

	if (!MoveFileEx(szModule, szBuffer, MOVEFILE_REPLACE_EXISTING))
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/aa365240.aspx

	if (!MoveFileEx(szProcess, szModule, MOVEFILE_CREATE_HARDLINK))
		return FALSE;
#endif
	return TRUE;
}

// https://msdn.microsoft.com/en-us/library/3y1sfaz2.aspx

__declspec(dllexport)
const	WCHAR	szConsole[] = L"com";

__declspec(dllexport)
const	WCHAR	szWindows[] = L"exe";
<<NOKEEP

MYTH0.EXE: MYTH0E.OBJ MYTH0.LIB # MYTHS.DOS
	LINK.EXE /LINK /BASE:65536 /BREPRO /DEFAULTLIB:KERNEL32.LIB /ENTRY:wWinMainCRTStartup /FIXED /LARGEADDRESSAWARE /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:4.0 /OUT:$@ /RELEASE /STUB:MYTHS.DOS /SUBSYSTEM:WINDOWS /SWAPRUN:CD,NET /VERSION:0.815 $**

MYTH0E.OBJ: MYTHS.MAK
	CL.EXE /Brepro /c /Fo$@ /GAF /Gwy /nologo /O2isy /Tc<< /W4 /we4013 /Zl
// Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

// https://msdn.microsoft.com/en-us/library/aa383732.aspx

#define STRICT
#define UNICODE
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

// https://msdn.microsoft.com/en-us/library/3y1sfaz2.aspx

__declspec(dllimport)
extern	WCHAR	szWindows[];

// https://msdn.microsoft.com/en-us/library/f9t8842e.aspx

__declspec(noreturn)
VOID	CDECL	wWinMainCRTStartup(VOID)
{
	WCHAR	szBuffer[MAX_PATH];
	WCHAR	szProcess[MAX_PATH];

	if ((GetModuleFileName((HMODULE) NULL, szProcess, sizeof(szProcess) / sizeof(*szProcess)) == 0UL)
	 || (GetTempPath(sizeof(szBuffer) / sizeof(*szBuffer), szBuffer) == 0U)
	 || (GetTempFileName(szBuffer, szWindows, 0U, szBuffer) == 0U)
	 || !DeleteFile(szBuffer)
	 || !MoveFile(szProcess, szBuffer))
		ExitProcess(GetLastError());

	ExitProcess(ERROR_SUCCESS);
}
<<NOKEEP

MYTH0.COM: MYTH0C.OBJ MYTH0.LIB # MYTHS.DOS
	LINK.EXE /LINK /BASE:65536 /BREPRO /DEFAULTLIB:KERNEL32.LIB /DEFAULTLIB:USER32.LIB /ENTRY:wmainCRTStartup /FIXED /LARGEADDRESSAWARE /MERGE:.rdata=.const /MERGE:.text=.code /NOCOFFGRPINFO /NOLOGO /NXCOMPAT /OPT:REF /OSVERSION:4.0 /OUT:$@ /RELEASE /STUB:MYTHS.DOS /SUBSYSTEM:CONSOLE /SWAPRUN:CD,NET /VERSION:0.815 $**

MYTH0C.OBJ: MYTHS.MAK
	CL.EXE /Brepro /c /Fo$@ /GAF /Gwy /nologo /O2isy /Tc<< /W4 /we4013 /Zl
// Copyright (C) 2004-2024, Stefan Kanthak <stefan.kanthak@nexgo.de>

// https://msdn.microsoft.com/en-us/library/7f0aews7.aspx

#pragma comment(compiler)

// https://msdn.microsoft.com/en-us/library/aa383732.aspx

#define STRICT
#define UNICODE
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

// https://msdn.microsoft.com/en-us/library/3y1sfaz2.aspx

__declspec(dllimport)
extern	WCHAR	szConsole[];

// https://msdn.microsoft.com/en-us/library/hh916382.aspx

__declspec(safebuffers)
BOOL	CDECL	PrintConsole(HANDLE hConsole, [SA_FormatString(Style="printf")] LPCWSTR lpFormat, ...)
{
	WCHAR	szOutput[1024];
	DWORD	dwOutput;
	DWORD	dwConsole;

	// https://msdn.microsoft.com/en-us/library/kb57fad8.aspx

	va_list	vaInput;
	va_start(vaInput, lpFormat);

	// https://msdn.microsoft.com/en-us/library/ms647551.aspx

	dwOutput = wvsprintf(szOutput, lpFormat, vaInput);

	va_end(vaInput);

	if (dwOutput == 0UL)
		return FALSE;

	// https://msdn.microsoft.com/en-us/library/ms687401.aspx

	if (!WriteConsole(hConsole, szOutput, dwOutput, &dwConsole, NULL))
		return FALSE;

	return dwConsole == dwOutput;
}

// https://msdn.microsoft.com/en-us/library/f9t8842e.aspx

__declspec(noreturn)
VOID	CDECL	wmainCRTStartup(VOID)
{
	WCHAR	szProcess[MAX_PATH];
	DWORD	dwProcess;
	WCHAR	szBuffer[MAX_PATH];
	DWORD	dwBuffer;
	DWORD	dwError = ERROR_SUCCESS;

	// https://msdn.microsoft.com/en-us/library/ms683231.aspx

	HANDLE	hConsole = GetStdHandle(STD_ERROR_HANDLE);

	if (hConsole == INVALID_HANDLE_VALUE)
		dwError = GetLastError();
	else
	{	// https://msdn.microsoft.com/en-us/library/ms683197.aspx

		dwProcess = GetModuleFileName((HMODULE) NULL,
		                              szProcess,
		                              sizeof(szProcess) / sizeof(*szProcess));
		if (dwProcess == 0UL)
			PrintConsole(hConsole,
			             L"GetModuleFileName() returned error %lu\n",
			             dwError = GetLastError());
		else
		{	// https://msdn.microsoft.com/en-us/library/aa364992.aspx

			dwBuffer = GetTempPath(sizeof(szBuffer) / sizeof(*szBuffer), szBuffer);

			if (dwBuffer == 0UL)
				PrintConsole(hConsole,
				             L"GetTempPath() returned error %lu\n",
				             dwError = GetLastError());
			else
				// https://msdn.microsoft.com/en-us/library/aa364991.aspx

				if (GetTempFileName(szBuffer, szConsole, 0U, szBuffer) == 0U)
					PrintConsole(hConsole,
					             L"GetTempFileName() returned error %lu\n",
					             dwError = GetLastError());
				else
					// https://msdn.microsoft.com/en-us/library/aa363915.aspx

					if (!DeleteFile(szBuffer))
						PrintConsole(hConsole,
						             L"DeleteFile() returned error %lu\n",
						             dwError = GetLastError());
					else
						// https://msdn.microsoft.com/en-us/library/aa365239.aspx

						if (!MoveFile(szProcess, szBuffer))
							PrintConsole(hConsole,
							             L"MoveFile() returned error %lu\n",
							             dwError = GetLastError());
		}

		// https://msdn.microsoft.com/en-us/library/ms724211.aspx

		if (!CloseHandle(hConsole))
			PrintConsole(hConsole,
			             L"CloseHandle() returned error %lu\n",
			             GetLastError());
	}

	// https://msdn.microsoft.com/en-us/library/ms682658.aspx

	ExitProcess(dwError);
}
<<NOKEEP
